import argparse
import sys
import logging

from postinstall import register
from constants import PrecheckPlugin
from constants import FORMAT
from constants import DATEFMT
from precheck.plugin_base import PluginBase

# initialize plugins for precheck
PRECHECK_PLUGINS = {p.id: p for p in PluginBase.plugins}

# Set up Logging
logging.basicConfig(format=FORMAT, datefmt=DATEFMT, level=logging.INFO)
logger = logging.getLogger(__name__)

def main():
    parser = argparse.ArgumentParser(description='Python utilities module for NDR')
    parser.add_argument(
        '--register-ndr',
        action="store_true",
        required=False,
        help="Invoke platform API to register NDR"
    )
    parser.add_argument(
        '--unregister-ndr',
        action="store_true",
        required=False,
        help="Invoke platform API to unregister NDR"
    )
    parser.add_argument(
        '--validate-license',
        action="store_true",
        required=False,
        help="Invoke API on NSX manager to retrieve licenses and "
        "validate if minimum requirements are met"
    )
    parser.add_argument(
        '--validate-platform-type',
        action="store_true",
        required=False,
        help="Invoke platform API  to validate if minimum platform "
        "form factor requirement is satisfied"
    )
    parser.add_argument(
        '--validate-capacity',
        action="store_true",
        required=False,
        help="Invoke platform API to validate that memory and CPU "
        "capacity requirements are satisfied"
    )
    args = parser.parse_args()

    if args.register_ndr:
        logging.info("Registering NDR feature.")
        register.ui_plugin()
    elif args.unregister_ndr:
        logging.info("Unregister NDR feature")
        register.ui_plugin(unregister=True)
    elif args.validate_license:
        license_plugin = PRECHECK_PLUGINS[PrecheckPlugin.LICENSE]()
        license_plugin.check()
    elif args.validate_platform_type:
        platform_type_plugin = PRECHECK_PLUGINS[PrecheckPlugin.PLATFORM_TYPE]()
        platform_type_plugin.check()
    elif args.validate_capacity:
        platform_type_plugin = PRECHECK_PLUGINS[PrecheckPlugin.CAPACITY]()
        platform_type_plugin.check()
    else:
        parser.print_help()

try:
    main()
except Exception as e:
    logging.exception(e)
    sys.exit(1)
