import os
import logging

from constants import CheckStatus
from constants import Feature
from constants import PrecheckPlugin
from constants import Url
from constants import CLUSTER_API_HOST
from constants import MONITOR_API_HOST
from constants import PlatformFormFactor
from precheck.plugin_base import PluginBase
from utils import requests_wrapper


class PluginCapacity(PluginBase):
    id = PrecheckPlugin.CAPACITY
    name = "feature.precheck.capacityName"
    desc = "feature.precheck.capacityDesc"
    feature = Feature.NDR

    def check(self):
        try:
            self.report_check_status(CheckStatus.INPROGRESS, "")

            # check if this is an EVAL FF
            monitor_api_port = os.environ["MONITOR_SERVICE_PORT"]
            url = Url.InCluster.GET_PLATFORM_STATUS.format(
                service_host=MONITOR_API_HOST,
                service_port=monitor_api_port)
            print("Calling Monitor service to get platform information")
            ff_response = requests_wrapper.incluster_request('GET', url)
            ff_json_response = ff_response.json()

            # create request parameters
            cluster_api_port = os.environ["CLUSTER_API_SERVICE_PORT"]
            url = Url.InCluster.CAPACITY_URL.format(
                     service_host=CLUSTER_API_HOST,
                     service_port=cluster_api_port)

            if ff_json_response["form_factor"] == PlatformFormFactor.EVALUATION:
                # EVAL FF is tight on resources - 25% buffer does not work
                # drop it to 10%. alarms are OKAY - we're on the bleeding edge
                req_data = {
                    "remaining_cpu_percent": 10,
                    "remaining_memory_percent": 10
                }
            else:
                # STANDARD / ADVANCED FF are prod setups - alarms are NOT OKAY
                # 85% high usage alarm == 15% required at all times
                # 10% for on-demand/scheduled jobs = 15+10 == 25%
                req_data = {
                    "remaining_cpu_percent": 25,
                    "remaining_memory_percent": 25
                }
            logging.info("Calling cluster-api to check if capacity exists to deploy feature")
            response = requests_wrapper.incluster_request('POST', url, data=req_data)
            json_response = response.json()

            if json_response["can_deploy"]:
                # can deploy == true, mark success
                self.report_check_status(CheckStatus.SUCCESS, "feature.precheck.capacityPassReason")
            else:
                # can't deploy - return reason
                self.report_check_status(
                    CheckStatus.FAILED,
                    "feature.precheck.capacityFailReason: {reason}".format(reason=json_response["reason"]))

        except Exception as e:
            self.report_check_status(
                CheckStatus.FAILED,
                "feature.precheck.nappApiInvocationFailedReason")
            logging.exception(e)
            raise